/* com/syncbuilder/internal/sync/NativeSocketImpl.java:
*
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: NativeSocketImpl.java,v 1.2 1998/09/20 18:29:12 christ Exp $ */


package com.syncbuilder.internal.sync;

import java.io.*;

import com.syncbuilder.internal.core.calls;

/**
* The implementation details of the Sockets which use the pilot-link native sockets.<br>
* This is to be considered an intermediary solution until the Pure Java implementations
* are ready for prime-time.
*
* @author Tilo Christ
*/
public class NativeSocketImpl
	extends com.syncbuilder.sync.SocketImpl
{
	public boolean isOpen()
	{
		return (_sdesc != 0);
	}


	synchronized public int getVersion() 
		throws IOException
    { 
		return calls.pi_version(_sdesc); 
	}

	
	/**
	* "Tickle" the PalmPilot in order to keep the connection alive. 
	* This can be automated through startWatchdog.
	*
	* @see #startWatchdog
	*/
    synchronized public void tickle() 
		throws IOException
    { 
		calls.pi_tickle(_sdesc); 
	}

	
	/**
	* Tickle the PalmPilot automatically in specified intervals
	*
	* @see #tickle
	*/
	synchronized public void startWatchdog(int interval) 
		throws IOException
	{
		calls.pi_watchdog(_sdesc, interval); 
	}


	/**
	* Return the socket descriptor. This is for internal purposes only!
	* This descriptor is only meaningful to the native pilot-link code.
	* <p>
	* <b>This method will go away as soon as the DLP can offer all functionality
	* through java.io-streams.</b>
	*/
	public int getDescriptor()
	{
		return _sdesc;
	}

	/**
	* Read a specified number of bytes into the specified array.<br>
	*/
	public synchronized int read(byte[] data, int len) 
		throws IOException
	{ 
		return calls.pi_read(_sdesc, data, len); 
	}
	
	/**
	* Write a specified number of bytes from the specified array.
	*/
	public synchronized int write(byte[] data, int len) 
		throws IOException
	{ 
		return calls.pi_write(_sdesc, data, len); 
	}


	/**
	* Fill in the socket-descriptor of the supplied SocketImpl.
	* This implies that the SocketImpl is a NativeSocketImpl.
	* This method is only there in order to comply with the
	* SocketImpl-interface. The timeout parameter is not supported
	* and values other than 0 will lead to an IllegalArgumentException!
	*/    
	synchronized public void accept(com.syncbuilder.sync.SocketImpl impl, int timeout) 
		throws IOException
    { 
		if (!(impl instanceof com.syncbuilder.internal.sync.NativeSocketImpl))
			throw new IllegalArgumentException("impl is not a NativeSocketImpl");

		if (timeout != 0)
			throw new IllegalArgumentException("timeout != 0 is currently unsupported");

		((NativeSocketImpl)impl)._sdesc = calls.pi_accept(_sdesc); 
	}

	/**
	* Fill in the socket-descriptor of the supplied SocketImpl.
	* This implies that the SocketImpl is a NativeSocketImpl.
	*/    
	synchronized public void accept(com.syncbuilder.sync.SocketImpl impl) 
		throws IOException
    { 
		accept(impl, 0);
	}

	synchronized public void bind(String device) 
		throws IOException
	{ 
		calls.pi_bind(_sdesc, device); 
	}

    synchronized public void listen(int backlog) 
		throws IOException
    { 
		calls.pi_listen(_sdesc, backlog); 
	}


	synchronized public void close() 
		throws IOException, com.syncbuilder.sync.DlpException 
	{ 
		/* this method must be idempotent */
		if (this._sdesc != 0)
		{
			calls.pi_noendsync(_sdesc);
	    	calls.pi_close(_sdesc); 
		}
			
		_sdesc = 0;
	}

	/** Switch the SocketImpl into ServerSocket-mode */
	public void create()
		throws IOException
	{
		_sdesc = calls.pi_socket(0x51, 0x10, 0x51);
		calls.pi_noendsync(_sdesc); // A ServerSocket never needs the end-of-hotsync sequence
	}


	/**
	* Only NativeSocketImplFactory shall instantiate this class.
	*/
	public NativeSocketImpl()
	{
	}
	
	
	// This descriptor is only meaningful to the native pilot-link code.
	private int _sdesc = 0;
}
