/* com/syncbuilder/internal/sync/PilotInputStream.java:
*
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: PilotInputStream.java,v 1.1 1998/09/16 08:57:35 christ Exp $ */

package com.syncbuilder.internal.sync;

/**
* PilotInputStream is a specialization of java.io.InputStream which
* is capable of receiving data from the PalmPilot through an instance of
* SocketImpl (either serial or TCP/IP).
*
* @author Tilo Christ
*/
public class PilotInputStream
	extends java.io.InputStream
{
	public PilotInputStream( com.syncbuilder.sync.SocketImpl simpl )
	{
		_simpl = simpl;
	}


	/**
	* Read a single byte from the PalmPilot. This is highly inefficient.
	* Use bulk-reads instead.<br>
	* <b>CAUTION: The return value is currently not guaranteed to be accurate</b>
	*/
	public int read()
		throws java.io.IOException
	{
		byte[] single = new byte[1];
		int num_read = _simpl.read(single, 1);

		if (num_read == 0) // Does this mean EOF?
			return -1;

		return single[0] & 0xff;
	}


	/**
	* Read several bytes from the PalmPilot. This is a very efficient
	* bulk-read.<br>
	* <b>CAUTION: The return value is currently not guaranteed to be accurate</b>
	*/
	public int read(byte[] b)
		throws java.io.IOException
	{
		if (b == null)
			throw new NullPointerException();

		int num_read = _simpl.read(b, b.length);

		if (num_read == 0) // Does this mean EOF?
			return -1;
		
		return num_read;
	}


	/**
	* Read several bytes from the PalmPilot. This is a very efficient
	* bulk-read.<br>
	* <b>CAUTION: The return value is currently not guaranteed to be accurate</b>
	*/
	public int read(byte[] b, int off, int len)
		throws java.io.IOException
	{
		if (b == null)
			throw new NullPointerException();

		if (b.length < off+len)
			throw new IllegalArgumentException("Byte-array too short!");


		int num_read = 0;
		if (off == 0)
		{
			num_read = _simpl.read(b, len);
		}
		else
		{
			byte[] input = new byte[len];
			num_read = _simpl.read(input, len);
			
			if (num_read != 0)
				System.arraycopy(input, 0, b, off, num_read);
		}

		if (num_read == 0) // Does this mean EOF?
			return -1;
		
		return num_read;
	}
	
		
	private final com.syncbuilder.sync.SocketImpl _simpl;
}
