/* com/syncbuilder/internal/sync/PilotOutputStream.java:
*
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: PilotOutputStream.java,v 1.1 1998/09/16 08:57:35 christ Exp $ */

package com.syncbuilder.internal.sync;

/**
* PilotOutputStream is a specialization of java.io.OutputStream which
* is capable of sending data to the PalmPilot through a SocketImpl
* (either serial or TCP/IP).
*
* @author Tilo Christ
*/
public class PilotOutputStream
	extends java.io.OutputStream
{
	/**
	* Create a new PilotOutputStream. 
	*/
	public PilotOutputStream( com.syncbuilder.sync.SocketImpl simpl )
	{
		_simpl = simpl;
	}

	/**
	* Write a single byte. This is highly inefficient.
	* Use bulk-writes instead.
	*/
	public void write(int b)
		throws java.io.IOException
	{
		byte[] single = new byte[1];
		single[0] = (byte)b;
		_simpl.write(single, 1);
	}

	/**
	* Write the contents of an array of bytes. 
	* This is an efficient bulk-write.
	*/
	public void write(byte[] b)
		throws java.io.IOException
	{
		_simpl.write(b, b.length);
	}
	
	/**
	* Write the contents of a specified part of an array of bytes.
	* This is very efficient if the specified offset is 0, and slightly
	* less efficient otherwise.
	*/
	public void write(byte[] b, int off, int len)
		throws java.io.IOException
	{
		if (b.length < off+len)
			throw new IllegalArgumentException("Byte-array too short!");

		if (off == 0)
		{
			_simpl.write(b, len);
		}
		else
		{
			byte output[] = new byte[len];
			System.arraycopy(b, off, output, 0, len);
			_simpl.write(output, len);
		}
	}

	private final com.syncbuilder.sync.SocketImpl _simpl;
}
