/* com/syncbuilder/service/ServiceStateEvent.java:
*
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: ServiceStateEvent.java,v 1.4 1998/10/25 14:08:55 christ Exp $ */

package com.syncbuilder.service;

/**
* Events of this kind are used by a service to report its state.
* <br>
* They should be sent out when the state changes, or when the service
* is asked to report its state.
* <br>
* ServiceStateEvents are required to contain their source, and they
* may also contain a <em>parent</em>. The parent is used in a hierarchy
* of services, where a sub-service wants to express that it belongs to
* a super-service, by specifying the super-service as its parent. 
* <br>
* This should make it easier to develop a hierarchical listing of 
* service states.
* <p>
* Last but not least, ServiceStateEvents may contain a message, which 
* describes the circumstances more closely.
* <p>
* As you can tell from the choice of available methods, ServiceStateEvents
* are immutable.
*
* @author Tilo Christ
*/
public class ServiceStateEvent
    extends java.util.EventObject 
{
	/**
	* Construct an event with the specified parent (super-service), and the
	* specified ServiceMessage.
	*/
	public ServiceStateEvent(Object source, Object parent, int state, ServiceMessage msg)
	{
		super(source);
		this._state = state;
		this._parent = parent;
		this._msg = msg;
	}

	public Object getParent()
	{
		return _parent;
	}

	public int getState()
	{
		return _state;
	}

	public String getStateName()
	{
		return _state_names[_state];
	}

	public ServiceMessage getMessage()
	{
		return _msg;
	}

	public String toString()
	{
		return "<com.syncbuilder.service.ServiceStateEvent: source = " + source + " parent = " + ((_parent != null) ? _parent.toString() : "(none)") + " state = " + getStateName() + " message = " + ((_msg != null) ? _msg.toString() : "(none)") + ">";
	}

	public static final int RUNNING = 0;
	public static final int STOPPED = 1;
	public static final int SUSPENDED = 2;
			
	private final int _state;	
	private final Object _parent;
	private final ServiceMessage _msg;
	
	private final static String[] _state_names = {"RUNNING", "STOPPED", "SUSPENDED"};
}
