/* com/syncbuilder/service/ServiceStateQueueAdaptor.java:
*
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: ServiceStateQueueAdaptor.java,v 1.2 1998/09/28 20:15:24 christ Exp $ */

package com.syncbuilder.service;

import java.util.Vector;

/*
* CAUTION: This implementation only implements the specified interface, but does not
* decouple the source from the listeners. Blocking listeners will block the whole
* system! This will definitely go away!!!
*/

/**
* This class serves two purposes:<br>
* 1. It decouples the source of ServiceStateEvents from its listeners,
*    thus ensuring uninterrupted service, even if there is a problem with
*    event-delivery
* <br>
* 2. It offers multicasting, which makes it unneccessary for event sources
*    to implement their own version of multicasting.
* <p>
* The life-cycle of the adaptor begins with the invocation of the constructor
* and ends with the invocation of the destroy()-method. It is always up and
* running in between these two points in time (you do not need to start or 
* stop it, nor is this possible).
*
* @author Tilo Christ
*/
public class ServiceStateQueueAdaptor
	implements ServiceStateListener
{
	/**
	* Create a working ServiceStateQueueAdaptor. When this constructor
	* has finished the Adaptor is running and ready for delivery of
	* events.
	*/
	public ServiceStateQueueAdaptor()
	{
	}
	
	/**
	* This method is part of the ServiceStateListener interface. 
	* Events which are sent to this method are delivered to all
	* registered listeners. This is a non-blocking call!
	*/
	public void stateChanged(ServiceStateEvent sse)
	{
		_notifyStateChanged(sse);
	}
	
	/**
	* Register a listener with the adaptor.
	*/
	public synchronized void addServiceStateListener(ServiceStateListener ssl)
	{
		_listeners.addElement(ssl);
	}
	
	/**
	* Unregister a listener from the adaptor. The listener will still
	* receive all events which were scheduled for it by the time it was
	* unregistered.
	*/
	public synchronized void removeServiceStateListener(ServiceStateListener ssl)
	{
		_listeners.removeElement(ssl);
	}

	/**
	* Ask the adaptor to release its resources and to cease working.
	* Undelivered events may get lost. This call is idempotent!
	*/
	public void destroy()
	{
		_listeners = null;
	}

	protected void finalize()
		throws Throwable
	{
		this.destroy();
	}

	/**
	* Send an event to all registered listeners. This is a non-blocking call.
	*/
	private void _notifyStateChanged(ServiceStateEvent sse)
	{
		Vector l;
		synchronized(this) { l = (Vector)_listeners.clone(); }

		// Deliver the event to all the interested listeners
		for (int i = 0; i < l.size(); i++)
		{
			((ServiceStateListener)l.elementAt(i)).stateChanged(sse);
		}
	}

	private Vector _listeners = new Vector();
}
