/* com/syncbuilder/storage/Block.java:
*
* Copyright (C) 1997, 1998, Kenneth Albanowski
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: Block.java,v 1.2 1998/10/25 14:07:53 christ Exp $ */

package com.syncbuilder.storage;

import java.io.*;

/** 
 * This is a superclass for all classes which need to parse
 * formatted byte-data into object-attribute values and vice versa.
 * The format of the byte-data is typically identical to the
 * format that is natively used by some Palm device-application.
 *
 * @author Kenneth Albanowski, changes by Tilo Christ
 */ 
abstract public class Block 
	implements java.io.Serializable
{
	/**
	* Create an uninitialized <code>Block</code>.
	* This no-arg constructor is also req'd for the externalization to work.
	*/
	public Block() 
	{
		this.fill();
	}

	public Block(byte[] contents) 
	{
		this.unpack(contents);
	}

	abstract public void fill();

	/**
	* Turn the Block's attribute-values into a byte-array.
	* The array can be used with unpack or for transfer
	* to an application on the Pilot.
	*
	* @return the array of bytes 
	* @see #unpack
	*/
	public byte[] pack() 
	{
		return raw;
	}

	public void unpack(byte[] data) 
	{
		raw = data;
	}

	public String describe() 
	{
		return "raw='" + com.syncbuilder.util.Util.prettyPrint(raw);
	}

	public String toString() 
	{
		return "<" + this.getClass().getName() + " " + describe() + ">";
	}

	protected byte[] raw;
}


