/* com/syncbuilder/storage/DatabaseImplFactory.java:
*
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: DatabaseImplFactory.java,v 1.2 1998/12/09 09:54:05 christ Exp $ */

package com.syncbuilder.storage;

import java.util.Vector;
import java.util.Hashtable;

/**
* This class organizes a global list of known specializations of DatabaseImpl.
* <br>It can accept registrations of new DatabaseImpls and it can find previously
* registered DatabaseImpls through different criteria.
* <br>There is no need to register the DatabaseImpls of the built-in applications,
* because these are already known to the factory.
*
* @author Tilo Christ
*/
public class DatabaseImplFactory
{
	/**
	* This class shall not be instantiated!
	*/
	private DatabaseImplFactory()
	{
	}


	public static DatabaseImpl findDatabaseImplByName(String dbname)
	{
		Vector name_vector = (Vector)(_dbimpl_by_dbname.get(dbname));

		if (name_vector == null)
			return null;
		
		return (DatabaseImpl)name_vector.elementAt(0);
	}


	public static DatabaseImpl findDatabaseImplByCreator(Char4 creator)
	{
		Vector cid_vector = (Vector)(_dbimpl_by_cid.get(creator));
		
		if (cid_vector == null)
			return null;
		
		return (DatabaseImpl)cid_vector.elementAt(0);
	}

	/**
	* Register an instance of DatabaseImpl with the factory. The instances
	* will be registered through both their name and their creator ID. <br>
	* The very same instance will be returned later, when you ask the factory to
	* return a DatabaseImpl with the given name or creator. 
	* <p>
	* Any DatabaseImpl which you want to register through this method needs to
	* have a proper (non null) name and creator ID.
	* <p>
	* Some applications (e.g. the built-in applications) have their name and 
	* creator ID hard-coded into their DatabaseImpl. Other applications
	* (e.g. the &quot;generic&quot; application) do not have this information
	* hard-coded into their DatabaseImpl, but you first need to instantiate their
	* DatabaseImpl with the required information before you can register it through
	* this method.
	*/
	public static void registerDatabaseImpl(DatabaseImpl dbimpl)
	{
		String name = dbimpl.getDBName();
		Char4 cid = dbimpl.getCreatorID();

		Vector name_vector = (Vector)(_dbimpl_by_dbname.get(name));
		
		if (name_vector == null)
		{
			name_vector = new Vector();
			_dbimpl_by_dbname.put(name, name_vector);
		}
		
		Vector cid_vector = (Vector)(_dbimpl_by_cid.get(cid));
		
		if (cid_vector == null)
		{
			cid_vector = new Vector();
			_dbimpl_by_cid.put(cid, cid_vector);
		}
		
		name_vector.addElement(dbimpl);
		cid_vector.addElement(dbimpl);
	}

    private static Hashtable _dbimpl_by_dbname = new Hashtable(10);
    private static Hashtable _dbimpl_by_cid = new Hashtable(10);

    static 
	{
		registerDatabaseImpl(new com.syncbuilder.storage.appointment.DatabaseImpl());
		registerDatabaseImpl(new com.syncbuilder.storage.memo.DatabaseImpl());
		registerDatabaseImpl(new com.syncbuilder.storage.todo.DatabaseImpl());
		registerDatabaseImpl(new com.syncbuilder.storage.mail.DatabaseImpl());
		registerDatabaseImpl(new com.syncbuilder.storage.expense.DatabaseImpl());
		registerDatabaseImpl(new com.syncbuilder.storage.address.DatabaseImpl());
    }
}
