/* com/syncbuilder/storage/File.java:
*
* Copyright (C) 1997, 1998, Kenneth Albanowski
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: File.java,v 1.3 1998/12/09 09:52:02 christ Exp $ */

package com.syncbuilder.storage;

import java.io.IOException;

import com.syncbuilder.sync.Link;

/** 
 * A representation of a Palm OS database-image that is stored
 * as a file on the host-system.<p>
 * These files typically have the extension .PDB for record databases
 * and .PRC for resource databases.
 *
 * @author Kenneth Albanowski, changes by Tilo Christ
 */
public class File 
{
	public static File open(String name) 
		throws IOException  
	{
		return new File(name);
	}

	public static File create(String name, DBInfo info) 
		throws IOException  
	{
		return new File(name, info);
	}
	
	public void close() 
		throws IOException  
	{
		/* This function must be idempotent */
		pi_file_close();
	}
	
	public Record getRecord(int index) 
		throws IOException  
	{
		return pi_file_read_record(index, dbClass);
	}

	public Record getRecord(RecordID id) 
		throws IOException  
	{
		return pi_file_read_record_by_id(id, dbClass);
	}

	public void addRecord(Record newRecord) 
		throws IOException  
	{
		pi_file_append_record(newRecord);
	}

	public void addResource(Resource newResource) 
		throws IOException  
	{
		pi_file_append_resource(newResource);
	}

	public Record createRecord() 
		throws IOException  
	{
		return dbClass.createRecord();
	}

	public Record createRecord(RecordID id) 
		throws IOException  
	{
		return dbClass.createRecord(id);
	}

	public Resource createResource() 
		throws IOException  
	{
		return dbClass.createResource();
	}

	public Resource createResource(Char4 type, int id) 
		throws IOException  
	{
		return dbClass.createResource(type, id);
	}
		
	public Resource getResource(int index) 
		throws IOException  
	{
		return pi_file_read_resource(index, dbClass);
	}

	public void setAppBlock(AppBlock appblock) 
		throws IOException  
	{
		pi_file_set_app_info(appblock);
	}

	public void setSortBlock(SortBlock sortblock) 
		throws IOException  
	{
		pi_file_set_sort_info(sortblock);
	}

	public AppBlock getAppBlock() 
		throws IOException  
	{
		return pi_file_get_app_info(dbClass);
	}

	public SortBlock getSortBlock() 
		throws IOException  
	{
		return pi_file_get_sort_info(dbClass);
	}
	
	public DBInfo getDBInfo() 
		throws IOException  
	{
		return pi_file_get_info();
	}
	
	public void setDBInfo(DBInfo info) 
		throws IOException  
	{
		pi_file_set_info(info);
	}

	public int getRecords() 
		throws IOException  
	{
		return pi_file_get_entries();
	}
	
	public void install(Link link, int cardno) 
		throws IOException  
	{
		pi_file_install(link.getSocket().getDescriptor(), cardno);
	}

	public void retrieve(Link link, int cardno) 
		throws IOException  
	{
		pi_file_retrieve(link.getSocket().getDescriptor(), cardno);
	}

	public void merge(Link link, int cardno) 
		throws IOException  
	{
		pi_file_merge(link.getSocket().getDescriptor(), cardno);
	}


	protected void finalize() 
		throws IOException  
	{
		close();
	}
	

	private int _pf;
	private DatabaseImpl dbClass;

	
	private File(String name) 
		throws IOException 
	{
// FIXME
/*		_pf = 0;
		dbClass = (DatabaseImpl)DatabaseImplFactory.dbClasses.get(name);
		if (dbClass == null) {
			dbClass = DatabaseImplFactory.defaultDbClass;
		}
		System.err.println("dbClass = "+dbClass);
		pi_file_open(name);
		*/
	}
	
	private File(String name, DBInfo info)  
		throws IOException 
	{
// FIXME
/*		_pf = 0;
		dbClass = (DatabaseImpl)DatabaseImplFactory.dbClasses.get(name);
		if (dbClass == null) {
			dbClass = DatabaseImplFactory.defaultDbClass;
		}
		if (info != null)
			pi_file_create(name, info);
		else
			pi_file_open(name); */
	}
	

	private native void pi_file_create(String name, DBInfo info) throws IOException;
	private native void pi_file_open(String name) throws IOException;
	private native Record pi_file_read_record(int index, DatabaseImpl dbClass) throws IOException;
	private native Record pi_file_read_record_by_id(RecordID id, DatabaseImpl dbClass) throws IOException;
	private native Resource pi_file_read_resource(int index, DatabaseImpl dbClass) throws IOException;
	private native int pi_file_get_entries() throws IOException;
	private native DBInfo pi_file_get_info() throws IOException;
	private native AppBlock pi_file_get_app_info(DatabaseImpl dbClass) throws IOException;
	private native SortBlock pi_file_get_sort_info(DatabaseImpl dbClass) throws IOException;
	private native void pi_file_set_info(DBInfo info) throws IOException;
	private native void pi_file_set_app_info(AppBlock app) throws IOException;
	private native void pi_file_set_sort_info(SortBlock sort) throws IOException;
	private native void pi_file_append_record(Record rec) throws IOException;
	private native void pi_file_append_resource(Resource rsc) throws IOException;
	private native void pi_file_close() throws IOException;
	private native void pi_file_retrieve(int socket, int card) throws IOException;
	private native void pi_file_install(int socket, int card) throws IOException;
	private native void pi_file_merge(int socket, int card) throws IOException;
		
	static {
		System.loadLibrary("JavaPisock");
	}
}

