/* com/syncbuilder/storage/Pref.java:
*
* Copyright (C) 1997, 1998, Kenneth Albanowski
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: Pref.java,v 1.3 1998/12/09 09:58:57 christ Exp $ */

package com.syncbuilder.storage;

import java.io.*;

/**
* A representation of a preference-object on the Palm device.
* Each preference is uniquely identified by the Creator ID of the application to which it
* belongs, and an ID number. It also has a version number, but this is not used for identification.
* The pref's backup-flag says whether this pref requires backup during synchronization.<p>
* The contents of a pref are formatted bytes, with a layout that is native to the application.
*
* @author Kenneth Albanowski, changes by Tilo Christ
*/
abstract public class Pref 
	extends Block 
{
	/**
	* Create a fully initialized Pref-object.
	*/
	public Pref(byte[] contents, Char4 creator, int id, int version, boolean backup) 
		throws NoCreatorException
	{
		if (creator == null) 
			throw new NoCreatorException();

		this.id = id;
		this.creator = creator;
		this.version = version;
		this.backup = backup;

		this.unpack(contents);
	}

    /**
     * Returns information about this <code>Pref</code> object as a formatted <code>String</code>.
     * @return	a <code>String</code> representation of this <code>Pref</code> object.
     */	
	public String describe() {
		return " creator="+creator.toString()+", id="+id+", version="+version+
			", backup="+backup+", raw='"+ com.syncbuilder.util.Util.prettyPrint(raw)+"'";
	}

	/** The ID number of the Pref */
	public int id;
	/** The version number of the Pref */
	public int version;
	/** The backup-bit of the Pref */
	public boolean backup;
	/** The creator ID of the application which uses this Pref */
	public Char4 creator;
}
