/* com/syncbuilder/storage/appointment/Record.java:
*
* Copyright (C) 1997, 1998, Kenneth Albanowski
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: Record.java,v 1.2 1998/09/18 07:09:10 christ Exp $ */

package com.syncbuilder.storage.appointment;

import java.io.*;
import java.util.Date;

import com.syncbuilder.util.*;

/** 
 * A representation of a datebook database record.
 *
 * @author Kenneth Albanowski, massive changes by Tilo Christ
 */
public class Record 
	extends com.syncbuilder.storage.Record 
{
	/**
	* Create an appointment Record, filled with default-values
	*
	* @see #fill
	*/
	public Record() 
	{
		super();
	}

	public Record(byte[] contents, com.syncbuilder.storage.RecordID id, int index, int attr, int cat) 
	{
		super(contents, id, index, attr, cat);
	}

	/**
	* Turn the native format of a datebook-record into 
	* proper settings for this object's attributes.
	*
	* @param data the array of bytes in native format
	*/
	public void unpack(byte[] data)
	{
		if (data.length < 8)
			throw new java.lang.IllegalArgumentException("data-array too short!"); 

		int begin_hour = data[0] & 0xff;
		int begin_min  = data[1] & 0xff;
		int end_hour   = data[2] & 0xff;
		int end_min    = data[3] & 0xff;

		int date = ((data[4] & 0xff) << 8) + (data[5] & 0xff);

		int year  = (date >> 9) + 4;
  		int month = ((date >> 5) & 0xf) - 1;
  		int mday  = date & 31;

		if ((begin_hour == 0xff) && (begin_min == 0xff))
		{
			begin = new Date(year, month, mday, 0, 0, 0);						
			end = null;
		}
		else
		{
			begin = new Date(year, month, mday, begin_hour, begin_min, 0);						
			end = new Date(year, month, mday, end_hour, end_min, 0);						
		}

		int iflags = data[6] & 0xff;

		// data[7] is gap-fill

		int index = 8;

		if ((iflags & alarmFlag) != 0)
		{
			alarm = true;
			advance = data[index++] & 0xff;
			advanceUnits = 
				time.get( data[index++] & 0xff);						
		}
		else
		{
			alarm = false;
			advance = 0;
			advanceUnits =
				time.get( 0 );
		}					

		if ((iflags & repeatFlag) != 0)
		{
			repeatType = 
				repeat.get( data[index] & 0xff);
			index += 2;	

			date = ((data[index++] & 0xff) << 8) + (data[index++] & 0xff);

			if (date == 0xffff)
				repeatEnd = null;
			else
			{
				year  = (date >> 9) + 4;
  				month = ((date >> 5) & 0xf) - 1;
  				mday  = date & 31;

				repeatEnd = new java.util.Date(year, month, mday, 0,0,0);
			}

			repeatFrequency = data[index++] & 0xff;

			int on = data[index++] & 0xff;

			repeatDay = 0;
			repeatWeekdays = new boolean[7];

			for(int i=0; i<7; i++)
				repeatWeekdays[i] = false;

			if (repeatType == com.syncbuilder.storage.appointment.repeat.MonthlyByDay)
			{
				repeatDay = on;
			}
			else if(repeatType == repeat.Weekly)
			{
				for(int i=0; i<7; i++)
				{
					repeatWeekdays[i] = ((on & (1<<i)) != 0);
				}
			}

			repeatWeekStart = data[index] & 0xff;
			index += 2;
		}
		else
		{
			repeatType = repeat.get(0);
			repeatEnd = null;
			repeatFrequency = 0;
			repeatDay = 0;

			for(int i=0; i<7; i++)
			{
				repeatWeekdays[i] = false;
			}
			repeatWeekStart = 0;		
		}

		if ((iflags & exceptFlag) != 0)
		{
			int num_exceptions = ((data[index++] & 0xff) << 8) + (data[index++] & 0xff);
			exceptions = new java.util.Date[num_exceptions];				

			for (int i = 0; i<num_exceptions; i++)
			{
				date = ((data[index++] & 0xff) << 8) + (data[index++] & 0xff);
				year  = (date >> 9) + 4;
  				month = ((date >> 5) & 0xf) - 1;
  				mday  = date & 31;

				exceptions[i] = new java.util.Date(year, month, mday,0,0,0);
			}
		}
		else
		{
			exceptions = null;
		}

		if ((iflags & descFlag) != 0)
		{
			Index index2 = new Index();
			description = 
				PackUtils.getZeroTermString(data, index, index2);
			index = index2.getValue();
		}
		else
		{
			description = "";
		}

		if ((iflags & noteFlag) != 0)
		{
			note =
				PackUtils.getZeroTermString(data, index, null);
		}
		else
		{
			note = null;
		}
	}


	/**
	* Turn the settings of the Record's attributes into the native format of the
	* PalmPilot's Datebook-application.
	*/
	public byte[] pack()
	{
		// determine length of native byte-field
		int destlen = 8;
		
		if (alarm)
      		destlen += 2;
	    if (repeatType != repeat.None)
    	    destlen += 8;
        if (exceptions != null)
        	destlen += 2 + 2 * exceptions.length;
        if (note != null)
  	        destlen += note.length() + 1;

		// Datebook-Records ALWAYS have a description, which might be zero-length though.
	    destlen += description.length() + 1;
		
		byte[] buf = new byte[destlen];
		
		// pack attributes		
		if (end != null)
		{
			buf[0] = (byte)(begin.getHours());
			buf[1] = (byte)(begin.getMinutes());
			buf[2] = (byte)(end.getHours());
			buf[3] = (byte)(end.getMinutes());			
		}
		else
		{
			buf[0] = (byte)0xff;
			buf[1] = (byte)0xff;
			buf[2] = (byte)0xff;
			buf[3] = (byte)0xff;
		}

		{
		int d = ((begin.getYear() - 4) << 9) | 
			((begin.getMonth() + 1) << 5) |
				begin.getDate();
				
		buf[4] = (byte)((d >> 8) & 0xff);
		buf[5] = (byte)(d & 0xff);
		}

		int iflags = 0;
		int index = 8;

		if (alarm)
		{
			iflags |= alarmFlag;

			buf[index++] = (byte)advance;
			buf[index++] = (byte)(advanceUnits.getValue());
		}

		if (repeatType != repeat.None)
		{
			iflags |= repeatFlag;
			
			int on;
			
			if (repeatType == repeat.MonthlyByDay)
			{
				on = repeatDay;
			}
			else if (repeatType == repeat.Weekly)
			{
				on = 0;
				for (int i=0; i<7; i++)
				{
					if (repeatWeekdays[i])
					{
						on |= 1<<i;
					}
				}
			}
			else
			{
				on = 0;
			}
			
			buf[index++] = (byte)(repeatType.getValue());
			buf[index++] = 0;

			if (repeatEnd == null)
			{
				buf[index++] = (byte)0xff;
				buf[index++] = (byte)0xff;
			}
			else
			{
				int d = ((repeatEnd.getYear() - 4) << 9) | 
					((repeatEnd.getMonth() + 1) << 5) |
						repeatEnd.getDate();

				buf[index++] = (byte)((d >> 8) & 0xff);
				buf[index++] = (byte)(d & 0xff);
			}
			
			buf[index++] = (byte)repeatFrequency;
			buf[index++] = (byte)on;
			buf[index++] = (byte)repeatWeekStart;
			buf[index++] = 0;
		}

		if (exceptions != null)
		{
			iflags |= exceptFlag;
			
			int len = exceptions.length;
			buf[index++] = (byte)((len >> 8) & 0xff);
			buf[index++] = (byte)(len & 0xff);
			
			for(int i=0; i < len; i++)
			{
				java.util.Date date = exceptions[i];
				int d = ((date.getYear() - 4) << 9) | 
					((date.getMonth() + 1) << 5) |
						date.getDate();
				buf[index++] = (byte)((d >> 8) & 0xff);
				buf[index++] = (byte)(d & 0xff);
			}			
		}

		iflags |= descFlag;
		
		index = PackUtils.setZeroTermString(buf, index, description);

		if (note != null)
		{
			PackUtils.setZeroTermString(buf, index, note);

			iflags |= noteFlag;
		}

		buf[6] = (byte)iflags;
		buf[7] = (byte)0;  // gap-fill

		raw = buf;
		
		return buf;
   	}


	/**
	* Fill in the Record for a single appointment at the current date and time, with no description.
	*/
	public void fill() 
	{
		super.fill();

		exceptions = null;
		description = "";
		note = null;
		alarm = false;
		advance = 0;
		advanceUnits = time.Minutes;
		repeatType = repeat.None;
		repeatEnd = null;
		repeatFrequency = 1;
		repeatDay = 0;
		repeatWeekdays = new boolean[7];
		for (int i=0;i<7;i++)
			repeatWeekdays[i] = false;
		begin = new java.util.Date();
		end = new java.util.Date();
	}

    public String describe() 
	{
        StringBuffer c = new StringBuffer(super.describe());

		if (isDeleted())
		{
			c.append("  !!!DELETED!!!");
			return c.toString();
		}

		c.append("start=");
        c.append(begin.toString());
        c.append(", end=" + Util.prettyPrint(end));
        c.append(", note=" + ((note == null) ? "(null)" : note));
        c.append(", description=" + description );
        c.append(", exceptions=[");
        if (exceptions != null) {
	        for(int i=0;i<exceptions.length;i++) {
	        	if (i>0)
	        		c.append(",");
	        	c.append(exceptions[i].toString());
	        }
	    }
        c.append("], advance="+advance+", advanceUnits="+advanceUnits);
        c.append(", repeatType="+repeatType+", repeatEnd="+((repeatEnd==null)?"(null)":repeatEnd.toString()));
        c.append(", repeatFrequency="+repeatFrequency+", repeatWeekStart="+repeatWeekStart);
        c.append(", repeatDay="+repeatDay+", repeatWeekdays=[");
        for(int i=0;i<repeatWeekdays.length;i++) {
        	if (i>0)
        		c.append(",");
        	c.append(repeatWeekdays[i]);
        }
        c.append("]");

        return c.toString();
    }


	/**
	* Check two Records for equality. This is mostly for testing purposes.
	*/
	public boolean equals(Object obj)
	{
		if (obj == null)
			return false;

		if (!(obj instanceof com.syncbuilder.storage.appointment.Record))
			return false;

		if (super.equals(obj) == false)
			return false;

		com.syncbuilder.storage.appointment.Record other = (com.syncbuilder.storage.appointment.Record)obj;

		if (!description.equals(other.description))
			return false;

		if (note == null && other.note != null)
			return false;

		if (note != null && !note.equals(other.note))
			return false;

		if (!begin.equals(other.begin))
			return false;

		if (end == null && other.end != null)
			return false;

		if (end != null && !end.equals(other.end))
			return false;

		if (alarm != other.alarm)
			return false;

		if (advance != other.advance)
			return false;

		if (advanceUnits != other.advanceUnits)
			return false;

		if (repeatType != other.repeatType)
			return false;

		if (repeatEnd == null && other.repeatEnd != null)
			return false;

		if (repeatEnd != null && !repeatEnd.equals(other.repeatEnd))
			return false;

		if (repeatFrequency != other.repeatFrequency)
			return false;

		if (repeatDay != other.repeatDay)
			return false;

		if (repeatWeekStart != other.repeatWeekStart)
			return false;

		if (repeatWeekdays == null && other.repeatWeekdays != null)
			return false;

		if (repeatWeekdays != null)
		{
			if (other.repeatWeekdays == null)
				return false;

			if (repeatWeekdays.length != other.repeatWeekdays.length)
				return false;

			for (int i=0;i<repeatWeekdays.length;i++)
				if (repeatWeekdays[i] != other.repeatWeekdays[i])
					return false;
		}			

		if (exceptions == null && other.exceptions != null)
			return false;


		if (exceptions != null)
		{
			if (other.exceptions == null)
				return false;

			if (exceptions.length != other.exceptions.length)
				return false;

			for (int i=0;i<exceptions.length;i++)
				if (!(exceptions[i].equals(other.exceptions[i])))
					return false;
		}			

		return true;
	}

	/** The description of the appointment. Is never null */
	public String description;
	/** An extra note for this appointment. Is null if there is no note */
	public String note;

	public java.util.Date begin;
	public java.util.Date end;

	public boolean alarm;

	public int advance;
	public time advanceUnits;

	public repeat repeatType;
	public java.util.Date repeatEnd;
	public int repeatFrequency;
	public boolean[] repeatWeekdays;
	public int repeatDay;
	public int repeatWeekStart;

	public java.util.Date[] exceptions;

 
	/**
	* This customized serialization is neccessary in order to
	* properly restore the type-safe constants. Otherwise we
	* would have to use equals() instead of using '==', 
	* because deserialization creates new instances of these objects.
	* We replace them by the existing singletons.
	*/
 	private void readObject(java.io.ObjectInputStream in)
    	throws IOException, ClassNotFoundException
	{
		in.defaultReadObject();

		advanceUnits = time.get( advanceUnits.getValue() );
		repeatType = repeat.get( repeatType.getValue() );
	}
 
 
	// Flags for packing/unpacking
	private final int alarmFlag  = 64;
	private final int repeatFlag = 32;
	private final int noteFlag   = 16;
	private final int exceptFlag =  8;
	private final int descFlag   =  4; // Always set in the current implementation of the PalmPilot datebook
}
