/* com/syncbuilder/storage/generic/DatabaseImpl.java:
*
* Copyright (C) 1997, 1998, Kenneth Albanowski
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: DatabaseImpl.java,v 1.2 1998/12/09 09:57:39 christ Exp $ */

package com.syncbuilder.storage.generic;

import com.syncbuilder.storage.Char4;
import com.syncbuilder.storage.NoCreatorException;

/**
* This implementation of DatabaseImpl is not made for any specific application.
* Its factory-methods will return generic implementations of the requested entities.
*
* @author Tilo Christ
*/
public class DatabaseImpl
	extends com.syncbuilder.storage.DatabaseImpl
{
	public DatabaseImpl()
	{
	}
	
	public DatabaseImpl(Char4 creator, String dbname)
	{
		this._creator = creator;
		this._dbname = dbname;
	}

	public com.syncbuilder.storage.Record createRecord() 
	{ 
		return new Record(); 
	}

	public com.syncbuilder.storage.Record createRecord(byte[] contents, com.syncbuilder.storage.RecordID id, int index, int attr, int cat) 
	{ 
		return new Record(contents, id, index, attr, cat); 
	}
		
	public com.syncbuilder.storage.Resource createResource() 
	{ 
		return new Resource(); 
	}
	
	public com.syncbuilder.storage.Resource createResource(byte[] contents, com.syncbuilder.storage.Char4 type, int id, int index) 
	{ 
		return new Resource(contents, type, id, index); 
	}
	
	public com.syncbuilder.storage.Pref createPref(byte[] contents, int id, int version, boolean backup)
		throws NoCreatorException
	{ 
		return new Pref(contents, _creator, id, version, backup); 
	}
		
	public com.syncbuilder.storage.AppBlock createAppBlock()
	{ 
		return new AppBlock(); 
	}
	 
	public com.syncbuilder.storage.AppBlock createAppBlock(byte[] contents)
	{ 
		return new AppBlock(contents); 
	} 
	
	public com.syncbuilder.storage.SortBlock createSortBlock() 
	{
		return new SortBlock(); 
	}
	
	public com.syncbuilder.storage.SortBlock createSortBlock(byte[] contents) 
	{ 
		return new SortBlock(contents); 
	}

	/**
	* Get the creator ID of the database. 
	* The creator ID is a unique four-letter ID which is used to link a database
	* to the application which created it.
	*/
    public com.syncbuilder.storage.Char4 getCreatorID() 
	{ 
		return _creator; 
	}
		
	/**
	* Get the name of the database.
	* The name can have up to 31 characters.
	*/
    public String getDBName() 
	{
		return _dbname; 
	}

	private Char4 _creator = null;
	private String _dbname = null;
}
