/* com/syncbuilder/storage/memo/Record.java:
*
* Copyright (C) 1997, 1998, Kenneth Albanowski
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/

package com.syncbuilder.storage.memo;

import java.io.*;

import com.syncbuilder.util.*;

/** 
* A representation of a memo database record.
*
* @author Kenneth Albanowski, massive changes by Tilo Christ
*/

public class Record 
	extends com.syncbuilder.storage.Record 
{
	public Record() 
	{
		super();
	}

	public Record(byte[] contents, com.syncbuilder.storage.RecordID id, int index, int attr, int cat) 
	{
		super(contents, id, index, attr, cat);
	}

	public String getMemo()
	{
		return _memo;
	}

	public void setMemo(String text)
	{
		_memo = (text != null) ? text : "";
	}

	public void unpack(byte[] data)
	{
		if (data.length == 0)
			throw new java.lang.IllegalArgumentException("data-array too short!"); 

		_memo = PackUtils.getZeroTermString(data, 0, null);
	}
	
	public byte[] pack()
	{
		int destlen = _memo.length() + 1;
		
		byte[] buf = new byte[destlen];

		PackUtils.setZeroTermString(buf, 0, _memo);
		
		return buf;		
	}

	public void fill() 
	{
		super.fill();
		_memo = "";
	}

	public boolean equals(Object obj)
	{
		if (!(obj instanceof com.syncbuilder.storage.memo.Record))
			return false;
			
		Record memo_rec = (com.syncbuilder.storage.memo.Record)obj;
		
		return _memo.equals(memo_rec.getMemo());
	}

    public String describe() 
	{
        return "text '"+ Util.prettyPrint(_memo)+"', " + super.describe();
    }


	private String _memo;
}
