/* com/syncbuilder/storage/todo/Record.java:
*
* Copyright (C) 1997, 1998, Kenneth Albanowski
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: Record.java,v 1.2 1998/09/18 07:10:40 christ Exp $ */

package com.syncbuilder.storage.todo;

import java.io.*;
import com.syncbuilder.util.*;


/** 
 * A representation of a todo database record.
 *
 * @author Kenneth Albanowski, massive changes by Tilo Christ
 */
public class Record 
	extends com.syncbuilder.storage.Record 
{
	/**
	* Create a todo-Record, filled with default-values.
	*
	* @see #fill
	*/
	public Record() 
	{
		super();
	}

	public Record(byte[] contents, com.syncbuilder.storage.RecordID id, int index, int attr, int cat) 
	{
		super(contents, id, index, attr, cat);
	}


	/**
	* Turn the native format of a to-do-record into 
	* proper settings for this object's attributes.
	*
	* @param data the array of bytes in native format
	*/
	public void unpack(byte[] data)
		throws java.lang.IllegalArgumentException
	{
		if (data.length < 3)
			throw new java.lang.IllegalArgumentException("data-array too short!");

		// Extract due-date
		int date = ((data[0] & 0xff) << 8) + (data[1] & 0xff);

		if (date != 0xffff)
		{
			int year  = (date >> 9) + 4;
	  		int month = ((date >> 5) & 0xf) - 1;
  			int mday  = date & 31;
			due = new java.util.Date(year, month, mday, 0, 0, 0);						
		}
		else
		{
			due = null;
		}

		// Extract priority and completed
		priority = data[2] & 0xff;
		if ((priority & 0x80) != 0)
		{
			complete = true;
			priority &= 0x7f;			
		}
		else 
		{
			complete = false;
		}
		
		int index = 3;
		description = "";
		note = null;

		if (data.length == 3)
			return;

		Index index2 = new Index();

		description = PackUtils.getZeroTermString(data, index, index2);
		index = index2.getValue();

		if (index == data.length)  // So that note remains null
			return;
			
		note = PackUtils.getZeroTermString(data, index, null);
	}
	
	/**
	* Turn the settings of the Record's attributes into the native format of the
	* PalmPilot's To-Do-application.
	*/
	public byte[] pack()
	{
		int destlen = 5;
		if (description != null)
			destlen += description.length();
		if (note != null)
			destlen += note.length();
			
		byte[] buf = new byte[destlen];
		
		if (due == null)
		{
			buf[0] = (byte)0xff;
			buf[1] = (byte)0xff;
		}
		else
		{
			int d = ((due.getYear() - 4) << 9) | 
				((due.getMonth() + 1) << 5) |
					due.getDate();
				
			buf[0] = (byte)((d >> 8) & 0xff);
			buf[1] = (byte)(d & 0xff);
		}
		
		buf[2] = (byte)priority;
		
		if (complete)
			buf[2] |= (byte)0x80;
			
		int pos = 3;
		
		pos = PackUtils.setZeroTermString(buf, pos, description);

		PackUtils.setZeroTermString(buf, pos, note);

		return buf;
	}


	/**
	* Fill the Record with default-values.<br>
	* Empty description, priority 1, no due-date, not completed, no note
	*/
	public void fill() 
	{
		super.fill();

		priority = 1;
		complete = false;
		description = "";
		note = null;
		due = null;
	}

    public String describe() 
	{
        return  "description '"+ Util.prettyPrint(description)+
      ", note '"+ Util.prettyPrint(note)+
      ", due "+ Util.prettyPrint(due) +
      ", complete " + complete +
      ", priority " + priority + "', "+ super.describe();
    }


	public String description;
	public String note;
	public java.util.Date due;
	public boolean complete;
	public int priority;
}
