/* com/syncbuilder/sync/Link.java:
*
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: Link.java,v 1.2 1998/12/09 09:44:53 christ Exp $ */

package com.syncbuilder.sync;


/**
* A Link is a single connection between a Palm device and the host.<br>
* It contains all the information about the established connection. This
* includes information about the underlying transport (Socket, Dlp), as
* well as higher level information (client authentification).
*
* @see com.syncbuilder.sync.Dlp
* @see com.syncbuilder.sync.Socket
* @author Tilo Christ
*/
public class Link
{
	/**
	* Construct a new Link from a Socket
	*/
	public Link(Socket sock)
		throws java.io.IOException
	{
		this._sock = sock;
		this._dlp = new Dlp(this);

		_version = sock.getVersion();
	}

	public Dlp getDlp()
		{ return _dlp; }

	/**
	* Retrieve the underlying Socket of the Link.
	*
	* @deprecated You are discouraged from operating directly on Sockets.
	*/
	public Socket getSocket()
		{ return _sock; }

	/**
	* Get the protocol-version of the connection. This information can be 
	* used to distinguish devices running Palm OS 1.0 from those devices 
	* running Palm OS 2.0 and higher.<br>
	* Devices running Palm OS 1.0 have limitations and bugs and therefore need to
	* be treated specially in some respects.<br>
	* The classes in this framework iron out these differences in a way
	* that is completely transparent to the developer.
	*/
	public int getProtocolVersion()
	{
		return _version;
	}

	private Socket _sock;
	private Dlp _dlp;
	private int _version;
}

