/* com/syncbuilder/sync/Socket.java:
*
* Copyright (C) 1997, 1998, Kenneth Albanowski
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: Socket.java,v 1.1 1998/09/16 08:57:45 christ Exp $ */


package com.syncbuilder.sync;

import java.io.*;

/**
* A connected Socket. 
*
* @author Tilo Christ
*/
public class Socket
{
	public Socket()
	{
		_impl = (_factory != null) ? (_factory.createSocketImpl()) : (new com.syncbuilder.internal.sync.NativeSocketImpl());
	}

	Socket(SocketImpl impl)
	{
		_impl = impl;
	}	
	
	public InputStream getInputStream()
		throws IOException
	{
		return _impl.getInputStream();
	}


	public OutputStream getOutputStream()
		throws IOException
	{
		return _impl.getOutputStream();
	}


	public boolean isOpen()
	{
		return _impl.isOpen();
	}


	/**
	* This method may only be invoked by the DLP that opened this socket
	*/	
	synchronized void close() 
		throws IOException, DlpException 
	{ 
		_impl.close();
	}


	synchronized public int getVersion() 
		throws IOException
    { 
		return _impl.getVersion();
	}

	
	/**
	* "Tickle" the PalmPilot in order to keep the connection alive. 
	* This can be automated through startWatchdog.
	*
	* @see #startWatchdog
	*/
    synchronized public void tickle() 
		throws IOException
    { 
		_impl.tickle(); 
	}

	
	/**
	* Tickle the PalmPilot automatically with a reasonable default-interval
	*
	* @see #tickle
	*/
	synchronized public void startWatchdog()
		throws IOException
	{
		startWatchdog(7); // value of 7 has been taken from the dlpsh.c - program
	}
                                                		

	/**
	* Tickle the PalmPilot automatically in specified intervals
	*
	* @see #tickle
	*/
	synchronized public void startWatchdog(int interval) 
		throws IOException
	{
		_impl.startWatchdog(interval); 
	}


	/**
	* Return the socket descriptor. This is for internal purposes only!
	* This descriptor is only meaningful to the native pilot-link code.
	*/
	public int getDescriptor()
	{
		return _impl.getDescriptor();
	}


    /**
     * Set the factory for all client and server sockets.
	 * Unlike the Socket/ServerSocket classes in the JDK,
	 * this package uses the factory from Socket for both
	 * client and server sockets.
     */
	public static void setSocketFactory(SocketImplFactory factory)
		throws IOException
	{
		if (_factory != null)
			throw new IOException("factory already defined.");

		_factory = factory;
	}


    /**
     * Return the factory for all client and server sockets.
	 * Unlike the Socket/ServerSocket classes in the JDK,
	 * this package uses the factory from Socket for both
	 * client and server sockets.
     */
	static SocketImplFactory getSocketFactory()
	{
		return _factory;
	}


    /**
     * The factory for all client and server sockets.
	 * Unlike the Socket/ServerSocket classes in the JDK,
	 * this package uses the Socket._factory for both
	 * client and server sockets.
     */
    private static SocketImplFactory _factory;

	/** Only ServerSocket.accept() shall touch this field! */
	final SocketImpl _impl;
}
