/* com/syncbuilder/sync/SocketImpl.java:
*
* Copyright (C) 1997, 1998, Kenneth Albanowski
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: SocketImpl.java,v 1.2 1998/09/20 18:25:08 christ Exp $ */

package com.syncbuilder.sync;

import java.io.*;

/**
* This interface needs to be implemented in order to provide a special
* type of socket (native pilot-link socket, Network HotSync socket, 
* pure Java serial socket). It specifies all the operations which are
* required by <code>Socket</code> and <code>ServerSocket</code>.
*
* @author Tilo Christ
*/
public abstract class SocketImpl
{
// =============================== Socket ==============================================================

	public abstract boolean isOpen();

	public abstract int getVersion() 
		throws IOException;
	
	/**
	* "Tickle" the PalmPilot in order to keep the connection alive. 
	* This can be automated through startWatchdog.
	*
	* @see #startWatchdog
	*/
    public void tickle() 
		throws IOException
	{
	}
	
	/**
	* Tickle the PalmPilot automatically in specified intervals
	*
	* @see #tickle
	*/
	public void startWatchdog(int interval) 
		throws IOException
	{
	}


	public InputStream getInputStream()
		throws IOException
	{
		return new com.syncbuilder.internal.sync.PilotInputStream( this );
	}


	public OutputStream getOutputStream()
		throws IOException
	{
		return new com.syncbuilder.internal.sync.PilotOutputStream( this );
	}


	/**
	* Read a specified number of bytes into the specified array.<br>
	*/
	public abstract int read(byte[] data, int len) 
		throws IOException;
	
	/**
	* Write a specified number of bytes from the specified array.
	*/
	public abstract int write(byte[] data, int len) 
		throws IOException;

	/**
	* Return the socket descriptor. This is for internal purposes only!
	* This descriptor is only meaningful to the native pilot-link code
	* in NativeSocketImpl.
	* <p>
	* <b>This method will go away as soon as the DLP can offer all functionality
	* through java.io-streams.</b>
	*/
	public int getDescriptor()
	{
		throw new RuntimeException("getDescriptor not available!");
	}


// ============================= ServerSocket ==============================================================

	/** Switch the SocketImpl into ServerSocket-mode */
	public abstract void create()
		throws IOException;

    public abstract void accept(SocketImpl impl) 
		throws IOException;

    public abstract void accept(SocketImpl impl, int timeout) 
		throws IOException;

	public abstract void bind(String device) 
		throws IOException;

    public abstract void listen(int backlog) 
		throws IOException;


// ============================= Socket & ServerSocket ====================================================

	public abstract void close() 
		throws IOException, DlpException;


	protected void finalize() 
		throws Throwable
	{
		this.close();
	}
}
