/* com/syncbuilder/sync/SyncHandler.java:
*
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: SyncHandler.java,v 1.3 1998/10/22 07:45:50 christ Exp $ */

package com.syncbuilder.sync;


/**
* Implementations of this interface are used in conjunction with the SyncServer class.
* They contain all the application-specific knowlegde that is required to service incoming 
* connections.<br>
* The SyncServer will accept connection requests from PalmOS devices and then direct them
* at your implementation of this interface.<br>
* Only one instance of your implementation will be used to service all incoming connections,
* which may occur simultaneously.<br>
* This means that your implementation of this interface should either not use global 
* variables, or needs to carefully synchronize access to them.<br>
* <b>Basically, this is an adaption of JavaSoft's Servlet principle.</b> You might want to read
* the documentation that comes with JavaSoft's JSDK to gain a better understanding.
*
* @see com.syncbuilder.sync.SyncServer
* @author Tilo Christ
*/
public interface SyncHandler
{
	/**
	* Initialize the SyncHandler. This method is guaranteed to be called exactly once before the
	* SyncHandler is being used to handle incoming connections.
	*/
//	public void init(SyncHandlerConfig sync_config) // Not ready yet
	public void init()
		throws Exception;
	
	/**
	* Service an incoming connection. This method is invoked everytime a device needs to be serviced.
	*
	* @param link the Link which is associated with the connection
	*/
	public void service(Link link)
		throws Exception;
	
	/**
	* Free all resources. This method will be invoked when the SyncHandler is no longer needed. The
	* SyncHandler should free all the resources it possessed.
	*/
	public void destroy()
		throws Exception;
}

