/* com/syncbuilder/sync/SyncServer.java:
*
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id: SyncServer.java,v 1.5 1998/12/09 09:44:03 christ Exp $ */

package com.syncbuilder.sync;


/**
* A SyncServer is the heart of all applications which are supposed to handle
* several simultaneous connections from Palm devices. It continuously accepts
* connections and spawns a separate thread to handle each one.<br>
* Each of these threads runs an implementation of the SyncHandler-interface.
* The threads terminate when their SyncHandler is done.<br>
* The SyncServer can be controlled through ServiceControlEvents.
*
* @see com.syncbuilder.service.ServiceControlListener
* @see com.syncbuilder.service.ServiceControlEvent
* @author Tilo Christ
*/
public class SyncServer
	extends com.syncbuilder.service.ServiceBase
{
	/**
	* Construct a SyncServer from an existing ServerSocket. This ServerSocket needs
	* to be able to establish multiple simultaneous connections. Each connection
	* will be handled by one and the same instance of an implementation of the SyncHandler interface.
	* You will have to provide that instance.<br>
	* This principle will be familiar to you if you have worked with Servlets before.
	* <br>
	* The SyncServer will be inactive after its construction. You need to invoke the
	* run()-method to awake it. After that you should control it by sending ServiceControlEvents
	* to it.
	*
	* @see #run
	* @see com.syncbuilder.service.ServiceControlEvent
	*/
	public SyncServer(ServerSocket serversock, SyncHandler synchandler)
	{
		super();
	
		this._synchandler = synchandler;
		this._serversock = serversock;
	}

	protected void init()
		throws Exception
	{
		_synchandler.init();
	}

	protected void destroy()
		throws Exception
	{
		_synchandler.destroy();
	}

	protected void service()
		throws Exception
	{
		System.err.println("SyncServer.service()");
	
		Link link = null;
		try
		{
			link = _serversock.accept(TIMEOUT);
		}
		catch (java.io.IOException ioe)
		{
			link = null; // This must be changed!
		}

		if (link != null)
		{
			Thread handler_thread = new HandlerThread(link, _synchandler);
			addThread(handler_thread);
			handler_thread.start();
		}
	}	


	// Inner class to handle concurrent execution of several SyncHandlers.
	class HandlerThread
		extends java.lang.Thread
	{
		public HandlerThread(Link link, SyncHandler synchandler)
		{
			this._link = link;
			this._synchandler = synchandler;
		}
		
		public void run()
		{
			try
			{
				this._synchandler.service( this._link );
			}
			catch(Exception e)
			{
				// Sth. went wrong! This will have to be handled!!!
			}
		}
		
		private final Link _link;
		private final SyncHandler _synchandler;
	}


	// timeout for accept() in milliseconds. Affects reaction time of the SyncServer to 
	// control commands.
	private final static int TIMEOUT = 500; 
	
	private ServerSocket _serversock;
	private SyncHandler _synchandler;
}

