/* com/syncbuilderx/directory/LDIFCtx.java
*
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Library Public License V2.
* See the file COPYING.LIB for details.
*/
/* $Id$ */

package com.syncbuilderx.directory;

import javax.naming.*;
import javax.naming.directory.*;
import java.util.*;

/**
* A JNDI Service Provider that provides read-only access to LDIF files.
*
* @author Tilo Christ
*/
class LDIFCtx 
	implements DirContext 
{
	/**
	* Create an LDIF DirContext with the specified environment.
	*/
    LDIFCtx(Hashtable environment) 
	{
        _environment = (environment != null)
	    ? (Hashtable)(environment.clone()) 
	    : null;
    }

    public Object lookup(String name) 
		throws NamingException 
	{
        if (name.equals("")) 
		{
            // Asking to look up this context itself.  Create and return
            // a new instance with its own independent environment.
            return (new LDIFCtx(_environment));
        }
		
        Object answer = _bindings.get(name);
		
        if (answer == null) 
		{
            throw new NameNotFoundException(name + " not found");
        }
		
        return answer;
    }

    public Object lookup(Name name) 
		throws NamingException 
	{
        return lookup(name.toString()); 
    }

	public void bind(String name, Object obj, Attributes attrs)
		throws NamingException 
	{
		throw new OperationNotSupportedException();
	}	

	public void bind(Name name, Object obj, Attributes attrs)
		throws NamingException 
	{
		bind(name.toString(), obj, attrs);
	}	

    public void bind(String name, Object obj) 
		throws NamingException 
	{
		bind(name, obj, null);
    }

    public void bind(Name name, Object obj) 
		throws NamingException 
	{
        bind(name.toString(), obj);
    }
	

    public void rebind(String name, Object obj) 
		throws NamingException 
	{
		throw new OperationNotSupportedException();
    }

    public void rebind(Name name, Object obj) 
		throws NamingException 
	{
        rebind(name.toString(), obj);
    }

    public void unbind(String name) 
		throws NamingException 
	{
		throw new OperationNotSupportedException();
    }

    public void unbind(Name name) 
		throws NamingException 
	{
        unbind(name.toString());
    }

    public void rename(String oldname, String newname)
    	throws NamingException 
	{
		throw new OperationNotSupportedException();
    }

    public void rename(Name oldname, Name newname)
    	throws NamingException 
	{
        rename(oldname.toString(), newname.toString());
    }

    public NamingEnumeration list(String name)
    	throws NamingException 
	{
		throw new OperationNotSupportedException();
    }

    public NamingEnumeration list(Name name)
    	throws NamingException 
	{
        return list(name.toString());
    }

    public NamingEnumeration listBindings(String name)
    	throws NamingException 
	{
		throw new OperationNotSupportedException();
    }

    public NamingEnumeration listBindings(Name name)
    	throws NamingException 
	{
        return listBindings(name.toString());
    }

    public void destroySubcontext(String name) 
		throws NamingException 
	{
        throw new OperationNotSupportedException();
    }

    public void destroySubcontext(Name name) 
		throws NamingException 
	{
        destroySubcontext(name.toString());
    }

    public DirContext createSubcontext(String name, Attributes attrs)
    	throws NamingException 
	{
        throw new OperationNotSupportedException();
    }

    public DirContext createSubcontext(Name name, Attributes attrs) 
		throws NamingException 
	{
        return createSubcontext(name.toString(), attrs);
    }

    public Context createSubcontext(String name)
    	throws NamingException 
	{
        throw new OperationNotSupportedException();
    }

    public Context createSubcontext(Name name) 
		throws NamingException 
	{
        return createSubcontext(name.toString());
    }

    public Object lookupLink(String name) 
		throws NamingException 
	{
        return lookup(name);
    }

    public Object lookupLink(Name name) 
		throws NamingException 
	{
        return lookupLink(name.toString());
    }

    public NameParser getNameParser(String name)
    	throws NamingException 
	{
        return _parser;
    }

    public NameParser getNameParser(Name name) 
		throws NamingException 
	{
        return getNameParser(name.toString());
    }

    public String composeName(String name, String prefix)
    	throws NamingException 
	{
        Name result = composeName(new CompositeName(name),
                                  new CompositeName(prefix));
        return result.toString();
    }

    public Name composeName(Name name, Name prefix)
    	throws NamingException 
	{
        Name result = (Name)(prefix.clone());
        result.addAll(name);
        return result;
    }

    public Object addToEnvironment(String propName, Object propVal)
    	throws NamingException 
	{
        if (_environment == null) 
		{
            _environment = new Hashtable();
		} 
		
		return _environment.put(propName, propVal);
    }

    public Object removeFromEnvironment(String propName) 
    	throws NamingException 
	{
        if (_environment == null)
            return null;

		return _environment.remove(propName);
    }

    public Hashtable getEnvironment() 
		throws NamingException 
	{
		if (_environment == null) {
	    	// Must return non-null!!!
	    	_environment = new Hashtable();
		}

        return _environment;
    }

    public void close() 
		throws NamingException 
	{
		_environment = null;
		_bindings = null;
    }

	
	public Attributes getAttributes(Name name)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public Attributes getAttributes(String name)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public Attributes getAttributes(Name name, String[] attr_ids)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public Attributes getAttributes(String name, String[] attr_ids)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public DirContext getSchema(Name name)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}
	
	public DirContext getSchema(String name)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}
	
	public DirContext getSchemaClassDefinition(Name name)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}
	
	public DirContext getSchemaClassDefinition(String name)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}
	

	public void modifyAttributes(Name name, int mod_op, Attributes attrs)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public void modifyAttributes(String name, int mod_op, Attributes attrs)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public void modifyAttributes(Name name, ModificationItem[] mods)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public void modifyAttributes(String name, ModificationItem[] mods)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}


	public void rebind(Name name, Object obj, Attributes attrs)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public void rebind(String name, Object obj, Attributes attrs)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}


	public NamingEnumeration search(String name, Attributes matching_attributes)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public NamingEnumeration search(String name, Attributes matching_attributes, String[] attributes_to_return)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public NamingEnumeration search(String name, String filter_expr, Object[] filter_args, SearchControls cons)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public NamingEnumeration search(String name, String filter, SearchControls cons)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public NamingEnumeration search(Name name, Attributes matching_attributes)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public NamingEnumeration search(Name name, Attributes matching_attributes, String[] attributes_to_return)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public NamingEnumeration search(Name name, String filter_expr, Object[] filter_args, SearchControls cons)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}

	public NamingEnumeration search(Name name, String filter, SearchControls cons)
		throws NamingException
	{
        throw new OperationNotSupportedException();
	}


    private Hashtable _environment;
    private Hashtable _bindings = new Hashtable();
    private final static NameParser _parser = new LDIFNameParser();
}
