/* samplecode/LoggingSyncServerTest.java:
*
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Public License V2.
* See the file COPYING for details.
*/
/* $Id: LoggingSyncServerTest.java,v 1.1 1998/10/25 12:24:47 christ Exp $ */

import com.syncbuilder.sync.*;
import com.syncbuilder.device.*;
import com.syncbuilder.service.*;

/**
* This program demonstrates the usage of the com.syncbuilder.sync.SyncServer-class.
* It will not work through the PalmPilot's regular HotSync application, but only through
* the Network HotSync application which you will have to obtain separately from Palm
* Computing.
* <br>
* This program is an extension to the SyncServerTest sample-program. It demonstrates the
* usage of the ServiceStateEvents for the determination of the current state of the
* server, and how to control the server (start, stop, suspend, resume).
*
* @author Tilo Christ
*/
public class LoggingSyncServerTest
{
	public static void main(String[] args)
	{
		try
		{
		// Create a ServerSocket for use with Network HotSync.
		Socket.setSocketFactory( new NetworkHSSocketImplFactory() );
		ServerSocket ssock = new ServerSocket(null);

		// Create the SyncServer from the ServerSocket. The SyncServer
		// will serve all clients using SimpleHandler (see the class
		// SimpleHandler below). SimpleHandler will mostly print diagnostic
		// messages and read the UserInfo from the device.
		SyncServer sync_server = new SyncServer(ssock, new SimpleHandler());		

		// Create a listener-class and register it with the SyncServer.
		// The listener will simply write messages to System.err.
		sync_server.addServiceStateListener(new SimpleServiceStateListener());
		
		System.err.println("Initiate the Network HotSync procedures now."); 

		// The SyncServer starts running NOW!
		sync_server.start();

		System.err.println("Press <RETURN> to suspend the service");
		com.syncbuilder.util.Util.readLine(); 
		
		// The SyncServer is suspended
		sync_server.suspend();

		System.err.println("Press <RETURN> to resume the service");
		com.syncbuilder.util.Util.readLine(); 
		
		// The SyncServer is resumed
		sync_server.resume();

		System.err.println("Press <RETURN> to stop the service");
		com.syncbuilder.util.Util.readLine(); 
		
		// The SyncServer is shut down
		sync_server.stop();

		}
		catch(Throwable t)
		{
			t.printStackTrace();
		}
	}


}

	/**
	* This non-public class is used by the SyncServer to handle incoming
	* connections.
	*/
	class SimpleHandler
		implements com.syncbuilder.sync.SyncHandler
	{
		/** Invoked one time at start-up */
		public void init()
		{
			System.err.println("SimpleHandler.init()");
		}

		/** Invoked every time a device connects */		
		public void service(Link link)
			throws Exception
		{
			System.err.println("SimpleHandler.service() with " + link.toString() );
			
			Dlp dlp = link.getDlp();
			
			// Do it ten times, so it takes more time...
			for (int i=0; i < 10; i++)
			{
				UserInfo uinfo = dlp.getUserInfo();
				System.err.println( uinfo.toString() );
			}
			
			dlp.endHotSync();
		}

		/** Invoked one time at shutdown */		
		public void destroy()
		{
			System.err.println("SimpleHandler.destroy()");
		}
	}


	/**
	* This non-public class is used by the SyncServer to log changes in the
	* service's state. This very basic implementation will simply write
	* messages to System.err.
	*/
	class SimpleServiceStateListener
	implements ServiceStateListener
	{
		public void stateChanged(ServiceStateEvent sse)
		{
			System.err.println(sse.toString());	
		}	
	}
