/* samplecode/SyncServerTest.java:
*
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Public License V2.
* See the file COPYING for details.
*/
/* $Id: SyncServerTest.java,v 1.1 1998/09/20 16:55:17 christ Exp $ */

import com.syncbuilder.sync.*;
import com.syncbuilder.device.*;

/**
* This program demonstrates the usage of the com.syncbuilder.sync.SyncServer-class.
* It will not work through the PalmPilot's regular HotSync application, but only through
* the Network HotSync application which you will have to obtain separately from Palm
* Computing.
*
* @author Tilo Christ
*/
public class SyncServerTest
{
	public static void main(String[] args)
	{
		try
		{
		// Create a ServerSocket for use with Network HotSync.
		Socket.setSocketFactory( new NetworkHSSocketImplFactory() );
		ServerSocket ssock = new ServerSocket(null);

		// Create the SyncServer from the ServerSocket. The SyncServer
		// will serve all clients using SimpleHandler (see the class
		// SimpleHandler below). SimpleHandler will mostly print diagnostic
		// messages and read the UserInfo from the device.
		SyncServer sync_server = new SyncServer(ssock, new SimpleHandler());		
		
		System.err.println("Initiate the Network HotSync procedures now."); 

		// The SyncServer starts running NOW!
		sync_server.start();

		System.err.println("Press <RETURN> to stop the service");
		com.syncbuilder.util.Util.readLine(); 
		
		// The SyncServer is shut down
		sync_server.stop();

		}
		catch(Throwable t)
		{
			t.printStackTrace();
		}
	}


}

	/**
	* This non-public class is used by the SyncServer to handle incoming
	* connections.
	*/
	class SimpleHandler
		implements com.syncbuilder.sync.SyncHandler
	{
		/** Invoked one time at start-up */
		public void init()
		{
			System.err.println("SimpleHandler.init()");
		}

		/** Invoked every time a device connects */		
		public void service(Link link)
			throws Exception
		{
			System.err.println("SimpleHandler.service() with " + link.toString() );
			
			Dlp dlp = link.getDlp();
			
			// Do it ten times, so it takes more time...
			for (int i=0; i < 10; i++)
			{
				UserInfo uinfo = dlp.getUserInfo();
				System.err.println( uinfo.toString() );
			}
			
			dlp.endHotSync();
		}

		/** Invoked one time at shutdown */		
		public void destroy()
		{
			System.err.println("SimpleHandler.destroy()");
		}
	}

