/* samplecode/dlptest.java:
*
* Copyright (C) 1998, Tilo Christ
*
* This is free software, licensed under the GNU Public License V2.
* See the file COPYING for details.
*/
/* $Id: dlptest.java,v 1.2 1998/12/09 09:41:32 christ Exp $ */


import com.syncbuilder.sync.*;
import com.syncbuilder.storage.*;
import com.syncbuilder.device.*;

/**
* A test for the Palm OS DLP-functionality.
* This is also a sanity check for the new Pure Java
* Implementation of the DLP.
*
* @author Tilo Christ
*/
public class dlptest
{
	public static void main(String[] args)
	{
		try
		{
		String port = null;
		
		System.out.print("Port to use (\".\" means Network HotSync) [/dev/pilot]? ");
		System.out.flush();
		
		port = com.syncbuilder.util.Util.readLine();

		if (port.equals("")) 
			port = "/dev/pilot";

		if (port.equals("."))
		{
			Socket.setSocketFactory( new NetworkHSSocketImplFactory() );
			System.out.println("Initiate Network HotSync now."); 
		}
		else
		{
			System.out.println("Please hit the HotSync button now."); 
		}
		

		ServerSocket ssock = new ServerSocket(port);

		Link link = ssock.accept();
		Dlp dlp = link.getDlp();

//		dlp.getSocket().startWatchdog(); // Leads to strange failures

		System.out.println("The connection-protocol has a version of: " + link.getProtocolVersion());

		System.out.println("Information about the HotSync user will");
		System.out.println("be read from your device");
		System.out.println("as soon as you start the action.");
		System.out.println("Press <RETURN> to start the action...");
		com.syncbuilder.util.Util.readLine();

		UserInfo uinfo = dlp.getUserInfo();
		System.out.println( uinfo.toString() );

		System.out.println("The user-info should have been displayed now.");
		System.out.println("Press <RETURN> to continue...");
		com.syncbuilder.util.Util.readLine();
		
		System.out.println("The message on your Palm device should");
		System.out.println("switch from 'Identifying User' to 'Synchronizing'");
		System.out.println("as soon as you start the action.");
		System.out.println("Press <RETURN> to start the action...");
		com.syncbuilder.util.Util.readLine();

		dlp.switchMessage();
		
		System.out.println("The message should have switched now.");
		System.out.println("Press <RETURN> to continue...");
		com.syncbuilder.util.Util.readLine();

		System.out.println("The message on your Palm device should");
		System.out.println("switch from 'Synchronizing' to 'Synchronizing Datebook'");
		System.out.println("as soon as you start the action.");
		System.out.println("Press <RETURN> to start the action...");
		com.syncbuilder.util.Util.readLine();

		Database dbase = dlp.openStockDB(new com.syncbuilder.storage.appointment.DatabaseImpl());

		System.out.println("The message should have switched now.");
		System.out.println("Press <RETURN> to continue...");
		com.syncbuilder.util.Util.readLine();


		System.out.println("All " + dbase.getRecordCount() + " entries from your datebook will be read and displayed");
		System.out.println("as soon as you start the action.");
		System.out.println("Press <RETURN> to start the action...");
		com.syncbuilder.util.Util.readLine();

		for (int i = 0;; i++)
		{
			Record rec = dbase.getRecord(i);
			
			if (rec == null)
				break;
			
			System.out.println( rec.toString() );
		}

		System.out.println("The records should have been printed now.");
		System.out.println("Press <RETURN> to continue...");
		com.syncbuilder.util.Util.readLine();

		System.out.println("Two test entries will be added to your datebook at the current date");
		System.out.println("as soon as you start the action.");
		System.out.println("Press <RETURN> to start the action...");
		com.syncbuilder.util.Util.readLine();

		// This record will be sent to the device, but will be overwritten later on
		com.syncbuilder.storage.Record record1 = dbase.createRecord();
		((com.syncbuilder.storage.appointment.Record)record1).description = "Dinner at the Black House";
		dbase.putRecord(record1);

		// This record will be sent to the device, and will remain untouched
		com.syncbuilder.storage.Record record2 = dbase.createRecord();
		((com.syncbuilder.storage.appointment.Record)record2).description = "Dinner at the Red House";
		dbase.putRecord(record2);

		// This will overwrite the original content of record1 on the device
		((com.syncbuilder.storage.appointment.Record)record1).description = "Dinner at the White House";
		dbase.putRecord(record1);

		dbase.close();

		System.out.println("The records should have been added now with UID " + record1.getID().getValue() + " and " + record2.getID().getValue() + ".");
		System.out.println("The first one should read: Dinner at the White House");
		System.out.println("The second one should read: Dinner at the Red House");
		System.out.println("Press <RETURN> to continue...");
		com.syncbuilder.util.Util.readLine();

		System.out.println("A list of all databases on your device will be printed");
		System.out.println("as soon as you start the action.");
		System.out.println("Press <RETURN> to start the action...");
		com.syncbuilder.util.Util.readLine();

		int i = 0;
		DBInfo dbinfo = null;
		while((dbinfo = dlp.getDBInfo(i++, true, true, 0)) != null)
		{
			System.out.println(dbinfo.toString());
		}

		System.out.println("Press <RETURN> to continue...");
		com.syncbuilder.util.Util.readLine();

		dlp.endHotSync();

		}
		catch(Throwable t)
		{
			t.printStackTrace();
		}
	}
}

