/* $Id: AppointmentTest.java,v 1.3 1998/09/20 18:20:19 christ Exp $ */

import java.io.*;
import com.syncbuilder.storage.appointment.Record;
import com.syncbuilder.storage.appointment.AppBlock;

/**
* DejaGNU test-case for the correctness of the 
* com.syncbuilder.storage.appointment.* pack()/unpack() pair of methods
*
* @author Tilo Christ 
*/
public class AppointmentTest
{
	public static void main(String[] args)
	{
		boolean debug = false;

		if (args.length == 1)
		{
			if (args[0].equals("-debug"))
				debug = true;
				
			if (args[0].equals("-help"))
			{
				System.out.println( "\n\nAppointmentTest: Test-case for the pack()/unpack() pair of methods.\n");
				System.out.println( "Expects the file data/datebook.dat, which you can create with the");
				System.out.println( "datesucker-program.\n");
				System.out.println( "Parameters: -debug : Print extra debug information");
				System.out.println( "            <none> : Print POSIX 1003.3 compliant test-result for DejaGNU");
				System.out.println( "\n" );
				
				System.exit(0);
			}
		}

		FileInputStream istream = null;

		int i = 0;

		try
		{
			istream = new FileInputStream("data/datebook.dat");
        	ObjectInputStream p = new ObjectInputStream(istream);

			// Test packing/unpacking the AppBlock
			AppBlock appblock = (AppBlock)p.readObject();			
			byte[] raw = appblock.pack();

			AppBlock appblock2 = new AppBlock();
			appblock2.unpack(raw);
			
			if (!appblock.equals(appblock2))
			{
				System.out.println("FAIL: AppBlock differs");
				System.exit(0);
			}						

			// Test packing/unpacking every single Record
			for (i=0;;i++)
			{
				Record record = (Record)p.readObject();
				
				if (debug)
				{
					System.out.println("Original:\n" + record.getID() + " " + record.describe());
				}
				
				raw = record.pack();
				
				if (debug)
				{
					System.out.println("Processing #" + i + ". Native length: " + raw.length);
				}
				
				// Copy over all attributes of Record 
				Record compare = new Record();
				compare.setID(record.getID());
				compare.setIndex(record.getIndex());
				compare.setCategory(record.getCategory());
				compare.setDeleted(record.isDeleted());
				compare.setModified(record.isModified());
				compare.setBusy(record.isBusy());
				compare.setSecret(record.isSecret());
				compare.setArchived(record.isArchived());

				compare.unpack(raw);

				if (debug)
				{
					System.out.println("Reconstruction:\n" + compare.getID() + " " + compare.describe());
					System.out.println( com.syncbuilder.util.Util.hexPrint( raw ) );
				}
				
				if (!compare.equals(record))
				{
					System.out.println("FAIL: Record " + i + " differs.");
					System.exit(0);
				}
			}
 		}
		catch (java.io.EOFException eofe)
		{
			System.out.println("PASSED: " + (i-1) + " Records processed.");
		}
		catch (java.io.IOException ioe)
		{
			System.out.println("UNRESOLVED: " + ioe);
		}
		catch (Throwable t)
		{
			System.out.println("FAIL: " + t);
			t.printStackTrace();
		}
		finally 
		{
			try
			{
				if (istream != null)
					istream.close();
			}
			catch (java.io.IOException ioe)
			{
				System.out.println("UNRESOLVED: " + ioe);
			}
		} 
		
	}
}
